# Schedwi
# Copyright (C) 2011-2013 Herve Quatremain
#
# This file is part of Schedwi.
#
# Schedwi is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Schedwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""Module to create new jobsets."""

import sys
import getopt

from sqlalchemy.orm.exc import NoResultFound

import path
import set_options
from tables.job_main import job_main


def usage():
    """Print a usage message on STDOUT."""
    print _("""Usage: mkjs [OPTION]... JOBSET...
Create new jobsets.

Options:
  --enable=True|False     enable (True) or disable (False) the jobset.
  --host=HOSTNAME[:PORT]  default host for jobs inside the jobset.  If not set,
                          same as the parent jobset.
  --calendar=CAL_NAME     calendar name. If not set, same as the parent jobset.
  --start=TIME            start time (HH:MM).
  --command=COMMAND       default command to run for jobs inside the jobset.
                          If set to `-', no command by default - jobs will
                          complete immediately.  If not set, use the one
                          defined by the parent jobset.
  --parameter=PARAMETER   default parameter to pass to the command.  This
                          option can be specified several times to define more
                          than one parameter.  If not set, use the parameters
                          defined by the parent jobset.
  --return=RETURN_CODE    default return code from the command.  A return code
                          equal or below this value means success.  If not set,
                          use the one set by the parent jobset.
  --user=USERNAME         default username under which the command must be run.
                          If not set, same as the parent jobset.
  --stdout=FILE           default command output file (stored on the Schedwi
                          agent).  If not set, same as the parent jobset.
  --stderr=FILE           default command error file (stored on the Schedwi
                          agent).  If not set, same as the parent jobset.
  --detach=True|False     start the jobs in detach mode (True) or not (False).
                          In detach mode, Schedwi will not wait for the
                          completion of the job but immediately set the status
                          of the job on whether the job has successfully
                          started or not. The detach mode can be useful to
                          start daemons, services or long running processes.
                          If not set, same as the parent jobset.
  --manual=True|False     ask confirmation (True) to operators to start the
                          jobset or not (False).  If not set, same as the
                          parent jobset.
  --manual-cmd=COMMAND    for manual jobs, command/script to run to alert
                          operators that a job is waiting for a start
                          confirmation. The following environment variables
                          are set and can be used in the command/script:
                          SCHEDWI_TEMPLATE (the path to a text file that
                          contains the message to send to operators),
                          SCHEDWI_JOBPATH, SCHEDWI_JOBID, SCHEDWI_START_TIME,
                          SCHEDWI_START_TIME_EPOCH, SCHEDWI_TIME_LIMIT and
                          SCHEDWI_TIME_LIMIT_EPOCH.
                          If not set, same as the parent jobset.
  --duration=MINUTES      maximum number of minutes for the jobset to complete
                          once running.  If not set, same as the parent jobset.
  --retries=NUM           number of retries when the jobset fails.
                          If not set, same as the parent jobset.
  --retries-int=MINUTES   number of minutes between retries.
                          If not set, same as the parent jobset.
  --limit=MINUTES         number of minutes for the jobset to be started past
                          its start time (number of minutes or HH:MM)
                          If not set, same as the parent jobset.
  --user-env=True|False   default value to spefify to load (True) or not
                          (False) the user account environment (~/.profile or
                          ~/.bashrc for instance) on the remote host before
                          running the job command.  If not set, same as the
                          parent jobset.
  --addenv=ENV_NAME       add an environment group to the jobset.  This
                          option can be specified several times.  All the
                          sub-jobs and jobsets will inherit this group.
  --enableenv             enable the use of environment groups (default)
  --disableenv            do not use any environment group for this jobset.
  --addfile=HOSTNAME[:PORT],True|False,FILE     add a file to check before
                          starting the jobset.  The value is composed of
                          three fields separated by a comma.  The first one is
                          the remote agent hostname on which to check the file.
                          The second specifies if the file must be present
                          (True) or not (False).  The last one is the file or
                          directory to check.
  --description=TEXT      a description.
  -h, --help              display this help.
    """)


def mkjs(sql_session, current_cwd, current_cwd_cal, arguments):
    """Create new jobsets.

    Arguments:
    sql_session -- SQLAlchemy session
    current_cwd -- current working jobset (a path.Path object)
    arguments -- list of arguments given to the mkjs command (list
                 of jobs/jobsets)

    """
    # Parse the parameters
    try:
        optlist, args = set_options.parse_args(arguments)
    except getopt.GetoptError, err:
        sys.stderr.write(_("mkjs: ") + str(err) + "\n")
        return 1
    for o, a in optlist:
        if o in ("-h", "--help"):
            usage()
            return 0
    if not args:
        sys.stderr.write(_("mkjs: missing the jobset name to create\n"))
        return 1
    paths = list()
    for arg in args:
        basename, p = path.get_new_paths(sql_session, arg, current_cwd)
        paths.append([basename, p])
    if not paths:
        sys.stderr.write(_("mkjs: no such jobset\n"))
        return 1
    session = sql_session.open_session()
    for basename, dirs in paths:
        for p in dirs:
            # Check that the jobset to create does not already exist
            try:
                full_path = path.Path(session, basename, p)
            except NoResultFound:
                pass
            else:
                sys.stderr.write(_("mkjs: %s already exists\n") % full_path)
                sql_session.cancel_session(session)
                return 1
            job = job_main(parent=p.id[-1],
                           name=basename,
                           type=0,
                           enabled=1,
                           has_env=1,
                           description='',
                           x=0, y=0,
                           cal_id=0,
                           start_time=-1)
            session.add(job)
            if set_options.set(session, current_cwd, current_cwd_cal,
                               job, optlist) != 0:
                sql_session.cancel_session(session)
                return 1
    sql_session.close_session(session)
    return 0
